/*
	UnWindows 1.1
	border.c

	Richard L. Kline
	Rensselaer Polytechnic Institute
	December 1993

	This program monitors the mouse cursor's location and plays a sound
	whenever the cursor crosses an imaginary threshold around
	any of the edges of the screen.

	---
	Modification history:
	1.1 16 May 94 -	no changes
*/

# include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

char audio_specs_file[128];
Display	*dpy;
XtAppContext	appCtx;
int		scr_width;		/* width of the screen */
int		scr_height;		/* height of the screen */

#define CHECK_INTERVAL	200		/* time in ms between checks */
#define THRESHOLD	10		/* width of active screen border */  

void CheckPointer();

/* ARGSUSED */
void main(argc, argv)
    int argc;
    char **argv;
{
  Widget	toplevel;
  
  strcpy ( audio_specs_file, (char *) getenv ( "HOME" ) );
  strcat ( audio_specs_file, "/.un_sound" );
  
  toplevel = XtInitialize("main", "Borders", NULL, 0,
			  &argc, argv);
  
  dpy = XtDisplay(toplevel);
  appCtx = XtWidgetToApplicationContext(toplevel);

  scr_width = DisplayWidth(dpy,0) - THRESHOLD;
  scr_height = DisplayHeight(dpy,0) - THRESHOLD;

  /* make up the first timeout */
  XtAppAddTimeOut (appCtx, CHECK_INTERVAL, CheckPointer, NULL);

  /* off to the races */
  XtAppMainLoop(appCtx);

} /* main */


void CheckPointer (x)
     XtPointer x;
{
  Window	rep_root, rep_child;
  int		rep_rootx, rep_rooty;
  unsigned int	rep_mask;
  int		dx, dy;

  /*** VARIABLES FOR PLAY ***/
  
  int  		play_argc = 0;		/* number of args */
  char		*play_argv[10];		/* arguments */
  char		window[25];		/* used in file read */
  char		audio_file[75];		/* used in file read */
  char		volume[4];		/* used in file read */
  char		*vol_str = "-v";	/* used in file read */
  FILE		*fp;			/* used in file read */
  char		border[25];		/* set when a border is hit */
  int		i;
  
  border[0] = '?'; border[1] = '\0';

  XQueryPointer (dpy, RootWindow (dpy, DefaultScreen (dpy)), &rep_root,
		 &rep_child, &rep_rootx, &rep_rooty, &dx, &dy, &rep_mask);
  
  if ( rep_rooty <= THRESHOLD )
    strcpy(border,"Top_Border");
  if ( rep_rooty >= scr_height )
    strcpy(border,"Bottom_Border");
  if ( rep_rootx <= THRESHOLD )
    strcpy(border,"Left_Border");
  if ( rep_rootx >= scr_width )
    strcpy(border,"Right_Border");
  
  if ( rep_rooty <= THRESHOLD || rep_rooty >= scr_height ||
      rep_rootx <= THRESHOLD || rep_rootx >= scr_width ) {
    
    if ((fp = fopen(audio_specs_file,"r")) == NULL) 
      (stderr,"unable to open %s\n",audio_specs_file);
    else {
      while (fscanf(fp,"%s %s %s\n",window, audio_file, volume) != EOF ) {
	if ( strcmp(window,border) == 0 ) {
	  if ( strcmp(volume,"0") != 0 ) {
	    play_argv[0] = "playit";     play_argc = 0;
	    play_argv[1] = vol_str;      play_argc++;
	    play_argv[2] = volume;       play_argc++;
	    play_argv[3] = audio_file;   play_argc++;
	    play_argv[4] = "--";         play_argc++;
	    playit(play_argc,play_argv);
	  }
	}
      }
      fclose(fp);
    }
  }

  /* be sure to do this again in CHECK_INTERVAL ms */
  XtAppAddTimeOut (appCtx, CHECK_INTERVAL, CheckPointer, NULL);
  
} /* CheckPointer */
