/*
	UnWindows 1.1
	change_sounds.c

	Gary Ormsby
	Rensselaer Polytechnic Institute
	Fall 1991

	This is the interface to change the sounds in the
	.un_sound file.  Sounds can be added, deleted, or
	modified.  Sounds are bound to windows or screen edges.
	A sound is bound to a window by entering the name of the
	window.  A sound is bound to a screen edge by entering
	Top_Border, Bottom_Border, Left_Border, Right_Border.

	Minor revisions by Richard L. Kline
	December 1993

	The program now uses the .un_sound file in the user's
	home directory rather than ~ormsbyg/proj/audio_specs.
	The "quit" button was moved from the far left to the far
	right of the row of buttons.

	---
	Modification history:
	1.1 16 May 94 - no changes
*/

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Cardinals.h>
#include <math.h>
#include <X11/Shell.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Form.h>
#include <X11/cursorfont.h>


char audio_specs_file[128];

#include <sys/types.h>
#include <dirent.h>
#include <sys/time.h>
#include <sys/resource.h>


int	read_dir();
void	sort();
void	make_text_sound();
void	make_text_list();
void	makepopup();
void    display_set_members();
void    display_sound();
void    play_sound();
void    play_sound_proc();
void    reset_dir_proc();
void    save_sound_proc();
void    quit_proc();
void    play_selection();
void    remove_sound();
void	read_data();
void	move_pt();
void	change_width();
int	read_sound_file();
char	*find_sound();
char	*num_to_str();
int	str_to_num();

XtCallbackProc	delete_sound_proc();
XtCallbackProc	add_sound_proc();

XtAppContext appCtx;

String setnametrans =
        "<Motion>:                 select-start()\n\
        <Btn1Down>:                 select-end(PRIMARY, CUT_BUFFER0) display_set_members()\n\
        <Btn2Down>:                 select-end(PRIMARY, CUT_BUFFER0) play_selection()";

String setsoundtrans =
        "<Motion>:                 select-start()\n\
	<BtnDown>:                 select-end(PRIMARY, CUT_BUFFER0) display_sound()";


XtActionsRec sound_actionTable [] = {
	{"display_sound", display_sound},
};

XtActionsRec actionTable [] = {
        {"display_set_members", display_set_members},
        {"play_selection", play_selection}
};
XtTranslations  trans;                  /* translations */

int memselect[] = {
  XawselectLine, XawselectNull, XawselectNull, XawselectNull, XawselectNull, XawselectNull
};

int dialogselect[] = {
  XawselectNull
};

typedef	struct	Sound_struct	Sound_struct;
struct	Sound_struct {
	String	window;
	String	directory;
	String	sound;
	String	volume;
	String	index;

	String	old_window;
	String	old_directory;
	String	old_sound;
	String	old_volume;
	String	old_index;
};



#define	INITIAL_LEVEL	0.5
#define	MAXFILES	300
#define	MAXSOUNDS	75
#define	MAXSOUNDSLENGTH	2
#define	WINDLENGTH	45
#define DIRLENGTH	255
#define	VOLLENGTH	3
#define	LINELENGTH	500
#define	INDEXLENGTH	3

#define	MAXVOLUME	100
#define NZ NULL,ZERO

static	Widget		quit;
static	Widget		add_sound;
static	Widget		delete_sound;
static	Widget		save_sounds;
static	Widget		realign_colors;

static	Widget		label_example;
static	Widget		scroll_bar;
static	Widget		list;
static	Widget		text_list;
static	Widget		text_sound;
static	Widget		sound_dir;
Widget main_box;

Widget		popup;           /* the popup parameter window */
Widget		junk;
Widget		index_dialog;
Widget		window_dialog;
Widget		sound_dialog;
Widget		volume_dialog;
Widget		play;



String		label_font_string = "-b&h-lucidatypewriter-bold-r-normal-sans-*-350-*-*-*-*-iso8859-*";

XFontStruct	*label_font;


Colormap        cmap;
Boolean		delete_snd = False;

String		example = "Hi There";

Sound_struct	*sounds[MAXSOUNDS];
int		num_sounds;
String		sound_text_string;

Widget	topLevel = NULL;

String fallback_resources[] = {
	"*font:		-b&h-lucidatypewriter-bold-r-normal-sans-*-350-*-*-*-*-iso8859-*",
	"*resizable:			True",
        "*main_box.height:		550",
        "*main_box.width:		1000",
        "*main_box*label_example*fromVert:	scroll_bar",
        "*main_box*label_example*resizable:	True",
        "*label_example*x:	530",
        "*label_example*y:	20",

	"*input:                  True",
	"*showGrip:               off",
    	"*text_sound*preferredPaneSize: 200",
    	"*text_sound*editType:          read",
    	"*text_sound*scrollVertical:    always",
    	"*text_sound*scrollHorizontal:  never",
    	"*text_sound*autoFill:          on",

        "*thickness:                    25",
        "*scroll_bar*x:             380",
        "*scroll_bar*y:             75",
        "*scroll_bar*height:        80",
        "*scroll_bar*width:         500",

	"*bottomform.borderWidth:	0",

	"*popdown.fromHoriz:	ok",

	"*index_dialog.fromVert:	sound_dir",
	"*window_dialog.fromVert:	index_dialog",
	"*sound_dialog.fromVert:	window_dialog",
	"*volume_dialog.fromVert:	sound_dialog",

	"*text_list.fromHoriz:		dialogform",
	"*text_list.horizDistance:	350",
    	"*text_list*preferredPaneSize:	200",
    	"*text_list*editType:		read",
    	"*text_list*scrollVertical:	always",
    	"*text_list*scrollHorizontal:	never",
    	"*text_list*autoFill:		on",


	"*appDefaultsVersion:    1",

	"XFontSel.cursor:        left_ptr",
	"*allowShellResize:      true",

	"*quitButton.Label:      quit",
	"*quitButton.left:       chainLeft",
	"*quitButton.right:      chainLeft",

	"*selectButton.Label:       select",
	"*selectButton.fromHoriz:   quitButton",
	"*selectButton.left:        chainLeft",
	"*selectButton.right:       chainLeft",
	
        NULL,
};

Display         *dpy;      /* the display */


char	main_dir[DIRLENGTH] = "/usr/demo/SOUND/sounds";
char	*Empty_String = "Sound File is Empty";

struct	direct *main_list;
char	*text_str = NULL;
int	text_str_len = 0;
int	max_text_pixel_len = 0;
char	*main_items[MAXFILES];
int	num_main;
int	curindex = 0;

void main(argc, argv)
    unsigned int argc;
    char **argv;
{
  Widget choosefont_box;
  Widget box1;
  Widget text_box;
  
  int	argc_int = 0;
  char	*argv_int[10];
  int	i;
  Arg	arg[10];
  Arg	garg[10];
  XFontStruct	*font;


  strcpy ( audio_specs_file, (char *) getenv ( "HOME" ) );
  strcat ( audio_specs_file, "/.un_sound" );
  
  topLevel = XtAppInitialize(&appCtx, "XEyes2", NULL, 0,
			     &argc, argv, fallback_resources, NULL, 0);
  
  dpy = XtDisplay(topLevel);
  
  cmap = DefaultColormap(dpy,DefaultScreen(dpy));
  
  main_box =
    XtCreateManagedWidget("main_box",formWidgetClass,topLevel,NULL,0);
  
  /* the ADD SOUND button */
  
  XtSetArg(arg[0],XtNx,15);
  XtSetArg(arg[1],XtNy,37);
  add_sound = XtCreateManagedWidget("Add Sound", commandWidgetClass,
				    main_box, arg, 2);
  
  /* the DELETE SOUND button */
  
  XtSetArg(arg[0],XtNx,15);
  XtSetArg(arg[1],XtNy,147);
  XtSetArg(arg[2], XtNfromHoriz, add_sound);
  delete_sound = XtCreateManagedWidget("Delete Sound", commandWidgetClass,
				       main_box, arg, 3);
  
  /*
    this is a window that allows the user to change the sound (and
    its volume) bound an to an individual window (or screen edge)
    */
  
  makepopup(add_sound);
  
  /* the SAVE button */
  
  XtSetArg(arg[0],XtNx,15);
  XtSetArg(arg[1],XtNy,257);
  XtSetArg(arg[2], XtNfromHoriz, delete_sound);
  save_sounds = XtCreateManagedWidget("Save Sounds", commandWidgetClass,
				      main_box, arg, 3);
  
  /* the QUIT button */
  
  XtSetArg(arg[0], XtNfromHoriz, save_sounds);
  quit = XtCreateManagedWidget("Quit", commandWidgetClass,
			       main_box, arg, 1);
  
  XtAddCallback (quit, XtNcallback, quit_proc, NULL);
  XtAddCallback (save_sounds, XtNcallback, save_sound_proc, NULL);
  XtAddCallback (delete_sound, XtNcallback, delete_sound_proc, NULL);
  XtAddCallback (add_sound, XtNcallback, add_sound_proc, NULL);
  
  /* read the audio_specs file */
  num_sounds = read_sound_file();
  
  /* create the list of windows and their sounds */
  make_text_sound(True /* == Create text widget */ );

  XtRealizeWidget(topLevel);
  
  XtAppMainLoop(appCtx);
}

/***********************************************/

/*
        num_to_str - converts an int to a string

        global variables
                - none

        inputs  - num - the int to convert
        outputs - str - destination of the converted integer
*/
char	*num_to_str(num)
int     num;
{
char    str[9];
int     i = 0, index = 0;
int     power = 10000;

	if ( num == 0 )
		return("0");

        while ( power != 0) {
                i = num / power;
                num = num - i*power;
                if ( index > 0 || i != 0 ) {
                        str[index] =  toascii(i+'0');
                        index++;
                }
                power = power / 10;
        }
        str[index] = '\0';
	return (str);
}

/*
        str_to_num - converts a string to a int

        global variables
                - none

        inputs  - str - the character string to convert
        outputs - returns an integer
*/
int str_to_num(str)
char    str[];
{
        int i, n;

        n = 0;
        for (i = 0; str[i] >= '0' && str[i] <= '9'; ++i)

        n = 0;
        for (i = 0; str[i] >= '0' && str[i] <= '9'; ++i)
                n = 10 * n + str[i] - '0';
        return (n);
}

ConvertColor(w, color_name)
Widget w;
char * color_name;
{
  XrmValue from, to;

  from.size = strlen(color_name) + 1;
  from.addr = color_name;

/*
 * This conversion accepts a colorname from rgb.txt, or a #rrrgggbbb
 * rgb color definition, or the special toolkit strings "XtDefaultForeground"
 * and "XtDefaultBackground".
 */

  XtConvert(w, XtRString, (XrmValuePtr) &from, XtRPixel, (XrmValuePtr) &to);
  if (to.addr == NULL) {
      return(-1);
  }

  return( (int) *((Pixel *) to.addr) );
}


/*
	This procedure displays the popup window.  The user
	can enter the name of a window (or screen edge), bind
	a sound to it, and choose the volume with which to play
	the sound at.
*/
XtCallbackProc
add_sound_proc(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
Arg     args[10];
char	*empty_default = "        ";
char	*volume_default = "75";
char line[LINELENGTH];
char str[INDEXLENGTH];
int     n=LINELENGTH;
int begin, end;
int     index, pos;

XFontStruct *font;
Sound_struct	*p;

	p = (Sound_struct *) malloc( sizeof(Sound_struct) );

	p->window = (char *) malloc( strlen(empty_default) );
	strcpy(p->window, empty_default);

        p->directory = (char *) malloc( strlen(main_dir) );
	strcpy(p->directory, main_dir);

	p->volume = (char *) malloc( strlen(volume_default) );
	strcpy(p->volume, volume_default);

	p->sound = (char *) malloc( strlen(empty_default) );
	strcpy(p->sound, (char *) empty_default );

	p->index = (char *) malloc( strlen(num_to_str(num_sounds)) );
	strcpy(p->index,num_to_str(num_sounds));

	sounds[num_sounds] = p;

	curindex = num_sounds;

        XtSetArg(args[0], XtNvalue, num_to_str(num_sounds));
        XtSetValues(index_dialog, args, 1);
	num_sounds++;

        XtSetArg(args[0], XtNvalue, empty_default);
        XtSetValues(window_dialog, args, 1);
        change_width(window_dialog, empty_default);

        XtSetArg(args[0], XtNvalue, empty_default);
        XtSetValues(sound_dialog, args, 1);
        change_width(sound_dialog, empty_default);

        XtSetArg(args[0], XtNvalue, volume_default);
        XtSetValues(volume_dialog, args, 1);
        XtPopup(popup, XtGrabExclusive);
}


XtCallbackProc
delete_sound_proc(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
Arg     args[10];
String  selected_color = "gray71";
int     pixel;
int     i;

        if ( num_sounds == 0 )
                return;

        if ( (pixel = ConvertColor(widget, selected_color)) == -1 )
                fprintf(stderr,"Cannot convert color: %s\n", selected_color);
        else {
                i = 0;
                XtSetArg(args[i],XtNbackground, (Pixel) pixel); i++;
                XtSetValues(widget, args, i);
        }

        delete_snd = True;
}


/*
	This is a generic procedure to create dialog widgets.
*/
create_dialog(w, parent, neighbor, obj_class, value, label, height, width, oneline)
Widget  *w;
Widget  *parent;
Widget	neighbor;
char    obj_class[];
char    value[];
char    label[];
int     height;
int     width;
Boolean	oneline;

{
        int     n;
        Arg     args[10];

        Widget  tw;                     /* the Dialog widget's value */
        Widget  lw;                  /* the Dialog widget's label */

        /* the following variables are used to override the dialog widget's
           return key.  Instead of the cursor going to the next line, the
           return key signifies end of input and calls read_data().       */

        XtTranslations  trans;                  /* translations */

        String  newTranslations = "<Key>Return: read_data()";

	static XtActionsRec	actionTable[1] = { 
		{"read_data",	read_data}
	};

        XtAppAddActions(appCtx, actionTable, XtNumber(actionTable));

        trans = XtParseTranslationTable(newTranslations);

	n = 0;
        XtSetArg(args[n], XtNvalue, (XtArgVal) value); n++;
        XtSetArg(args[n], XtNlabel, (XtArgVal) label); n++;
        XtSetArg(args[n], XtNborderWidth, 0); n++;
        XtSetArg(args[n], XtNfromVert, neighbor); n++;
        XtSetArg(args[n], XtNselectTypes, (XtArgVal) dialogselect); n++;
        *w = XtCreateManagedWidget(obj_class, dialogWidgetClass, *parent, args, n);

        /* override the return key in the dialog box's AsciiText child */

        /*XtSetArg(args[n], XtNtranslations, &trans); n++;*/
        lw = XtNameToWidget(*w,"*label");
        tw = XtNameToWidget(*w,"*value");
        if ( tw !=  NULL) {
		n = 0;
        	XtSetArg(args[n], XtNresize, XawtextResizeWidth); n++;
        	XtSetArg(args[n], XtNborderWidth, 0); n++;
		if ( oneline ) {
        		XtSetArg(args[n], XtNfromHoriz, lw); n++;
        		XtSetArg(args[n], XtNfromVert, None); n++;
		}
                XtSetArg(args[n], XtNfromHoriz, lw); n++;
                XtSetArg(args[n], XtNfromVert, None); n++;
		XtSetValues(tw, args, n);
                XtOverrideTranslations(tw,trans);
	}
}

/*
	This procedure sorts the sounds.
*/
void	sort_sounds(number)
int		number;
{
  int     gap, i,j;
  Sound_struct    *temp;

  for (gap = number/2; gap>0; gap /= 2)
    for (i=gap; i<number; i++)
      for (j= i-gap; j>= 0; j -= gap) {
	if (str_to_num(sounds[j]->index) >
	    str_to_num(sounds[j+gap]->index))
	  break;
	temp = (Sound_struct *) sounds[j];
	sounds[j] = (Sound_struct *) sounds[j+gap];
	sounds[j+gap] = (Sound_struct *) temp;
      }
}


/*
	Verifies that the user entered a number for the index.
*/
int	chk_index(widget, str)
Widget	*widget;
char	*str[];
{
  Arg	  args[3];
  Boolean error=FALSE;
  int	  i;
  
  if ( str_to_num(str) > MAXSOUNDS ) {
    error = TRUE;
    return !error;
  }
  
  for (i=0; i < strlen(*str); i++) {
    if ( !isdigit((*str)[i]) ) {
      XtSetArg(args[0], XtNvalue, (XtArgVal) sounds[curindex]->index);
      XtSetValues(*widget, args,1);
      error = TRUE;
      break;
    }
  }
  if ( !error && str_to_num(*str) >= num_sounds)
    strcpy(*str, num_to_str(num_sounds-1));
  
  return !error;
}


/*
	Verifies that the user entered a valid number for the volume.
*/
int	chk_volume(widget, str)
Widget	*widget;
char	*str[];
{
  Arg	args[3];
  Boolean	error=FALSE;
  int	i;
  int	index;


  if ( strlen(*str) == 0 ) {
    error = TRUE;
    return  !error;
  }
  
  index = str_to_num ( XawDialogGetValueString(index_dialog) ); 
  for (i=0; i < strlen(*str); i++) {
    if ( !isdigit((*str)[i]) ) {
      XtSetArg(args[0], XtNvalue, (XtArgVal) sounds[index]->volume);
      XtSetValues(*widget, args,1);
      move_pt(*widget, strlen(sounds[index]->volume));
      error = TRUE;
      break;
    }
  }
  if ( !error && (str_to_num(*str) < 0 || str_to_num(*str) > MAXVOLUME)) {
    XtSetArg(args[0], XtNvalue, (XtArgVal) sounds[index]->volume);
    XtSetValues(*widget, args,1);
    move_pt(*widget, strlen(sounds[index]->volume));
    error = TRUE;
  }
  return  !error;
}


/*
	Verifies that a valid filename was entered for the sound.
*/
int     chk_sound(widget, str)
Widget  *widget;
char    *str[];
{
  Arg     args[3];
  Boolean allspaces=TRUE;
  int     i;
  char	c;
  
  for (i=0; i < strlen(*str); i++) {
    c = (char) ((*str)[i]);
    if ( !isspace(c) ) {
      allspaces=FALSE;
      break;
    }
  }
  return !allspaces;
}


/*
	Read the data from the dialog widgets in the popup window.
*/
void	read_data()
{
  int	i;

  String index_str = XawDialogGetValueString(index_dialog);
  String window_str = XawDialogGetValueString(window_dialog);
  String sound_str = XawDialogGetValueString(sound_dialog);
  String volume_str = XawDialogGetValueString(volume_dialog);
  
  if ( chk_index(	&index_dialog, &index_str ) ) {
    if (str_to_num(sounds[curindex]->index) != str_to_num(index_str))
      if (str_to_num(sounds[curindex]->index) < num_sounds) {
	Sound_struct    *temp = 
	  sounds[str_to_num(index_str)];
	sounds[str_to_num(index_str)] = 
	  sounds[ curindex ]; 
	sounds[ curindex ] = temp;
	strcpy(sounds[curindex]->index, num_to_str(curindex));
	strcpy(sounds[str_to_num(index_str)]->index,
	       index_str);	
      }
  }
  
  strcpy(sounds[str_to_num(index_str)]->window, window_str);
  
  if ( chk_volume( &volume_dialog, &volume_str ) ) {
    strcpy(sounds[str_to_num(index_str)]->volume, volume_str);
  }
  
  if ( chk_sound( &sound_dialog, &sound_str )) {
    int index = str_to_num(index_str);
    
    free (sounds[index]->sound);
    
    sounds[index]->sound = NULL;
    sounds[index]->sound = (char *) calloc( strlen(sound_str),1 );
    strcpy(sounds[index]->sound, sound_str);
  }
  
  free(sounds[str_to_num(index_str)]->directory);
  sounds[str_to_num(index_str)]->directory = (char *) calloc( strlen(main_dir)
		     + 1 + strlen(sounds[str_to_num(index_str)]->sound), 1);
  
  strcpy(sounds[str_to_num(index_str)]->directory,main_dir);
  strcat(sounds[str_to_num(index_str)]->directory,"/");
  strcat(sounds[str_to_num(index_str)]->directory,
	 sounds[str_to_num(index_str)]->sound);
}


/*
	Pop the popup widget down.
*/
static  XtCallbackProc
pdown(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
  Arg     args[10];


  read_data();
  make_text_sound(False);
  XtPopdown((Widget)closure);
}


/*
	Make the popup window that allows the user to
	manipulate a window's sound and the volume.
*/
void	makepopup(parent)
Widget	*parent;
{
  Arg             arg[10];
  Widget          pbox, ok, popdown, dialogform, topform, bottomform;
  Widget		record, reset_dir;
  Widget		set_window;
  Position        x, y;
  Dimension       width, height;
  Cardinal        n, i;
  char            str[5];
  
  /* set the size of the window */
  n = 0;
  XtSetArg(arg[n], XtNheight, 800);                         n++;
  XtSetArg(arg[n], XtNwidth, 800);                         n++;
  
  /* create the popup widget */
  popup = XtCreatePopupShell("prompt", transientShellWidgetClass, *parent,	
			     NULL, 0);
  
  /* create a box to hold everything */
  n = 0;
  XtSetArg(arg[n], XtNorientation, "vertical");	n++;
  pbox = XtCreateManagedWidget("pbox", formWidgetClass, popup, arg, n);
  
  popdown = XtCreateManagedWidget("Done", commandWidgetClass, pbox, NULL, 0);
  XtAddCallback(popdown,XtNcallback, pdown, (caddr_t)popup);
  
  n = 0;
  XtSetArg(arg[n], XtNfromHoriz,popdown); n++;
  play = XtCreateManagedWidget("Play Sound",commandWidgetClass,pbox,arg,n);
  
  XtAddCallback (play, XtNcallback, play_sound_proc, NULL);
  
  create_dialog(&sound_dir, &pbox, popdown, "sound_dir", main_dir, "Directory: ", 0,0, True);
  
  n = 0;
  XtSetArg(arg[n], XtNfromVert,sound_dir); n++;
  reset_dir = XtCreateManagedWidget("Read Directory",commandWidgetClass,pbox,arg,n);
  
  XtAddCallback (reset_dir, XtNcallback, reset_dir_proc);
  
  create_dialog(&index_dialog, &pbox, reset_dir, "index_dialog", "", "Index:  ", 0,0, False);
  
  create_dialog(&window_dialog,&pbox,index_dialog,"window_dialog","","Window: ",0,0, False);
  
  create_dialog(&sound_dialog,&pbox,window_dialog,"sound_dialog","","Sound:  ",0,0, False);
  
  create_dialog(&volume_dialog,&pbox,sound_dialog,"volume_dialog", "","Volume: ",0,0, False);
  
  num_main = read_dir(main_items, main_dir);
  
  /* sort the sounds */
  sort(main_items, num_main);
  
  /* put the sounds in a text widget so they can be viewed and selected */
  make_text_list(pbox, True);
}


/*
  Resets the directory where the sound files are.
*/
void
reset_dir_proc(w,cl,ca)
     Widget w;
     caddr_t cl,ca;
{
  int	i;
  
  for (i=0; i< num_main; i++) {
    free (main_items[i]); 
  }
  free ( text_str );
  
  strcpy(main_dir, XawDialogGetValueString(sound_dir));
  
  num_main = read_dir(main_items, main_dir);
  sort(main_items, num_main);
  
  make_text_list(XtParent(sound_dir), False);
}


/*
	Places windows, their sounds, and their volumes
	in a text widget for selection.
*/
void	make_text_list(parent,construct_text_widget)
     Widget	parent;
     Boolean	construct_text_widget;
{
   int		i, n;
   Arg		arg[10];
   XFontStruct	*font;
   
   font = (XFontStruct *) XLoadQueryFont( dpy, label_font_string);
   
   if ( font == NULL )
     printf ("font is null\n");
   
   text_str = NULL;
   for (i=0; i< num_main; i++) {
     char    *lf = "\n";
     char    *skip1 = ".";
     char    *skip2 = "..";
     int     width;
     
     if ( strcmp(main_items[i], skip1) == 0 )
       continue;
     if ( strcmp(main_items[i], skip2) == 0)
       continue;
     
     if ( text_str == NULL ) {
       text_str = (char *) malloc( strlen(main_items[i])+1);
       strcpy(text_str, main_items[i]);
     }
     else {
       text_str = (char *) realloc( text_str, strlen(text_str) +
				   strlen(main_items[i])+1);
       strcat(text_str, main_items[i]);
     }
     
     if ( max_text_pixel_len < (width = XTextWidth(font, main_items[i],
						   strlen(main_items[i])))  )
       max_text_pixel_len = width;
     
     strcat(text_str, "\n");
   }
   
   if ( construct_text_widget ) {
     i = 0;
     XtSetArg(arg[i],XtNx,60);       i++;
     XtSetArg(arg[i],XtNy,15);       i++;
     XtSetArg(arg[i],XtNheight,500);       i++;
     XtSetArg(arg[i],XtNwidth, max_text_pixel_len+30);       i++;
     XtSetArg(arg[i],XtNstring, text_str);   i++;
     XtSetArg(arg[i],XtNselectTypes, (XtArgVal)memselect);   i++;
     XtSetArg(arg[i], XtNfromVert, sound_dir); i++;
     XtSetArg(arg[i], XtNfromHoriz, index_dialog); i++;
     text_list = XtCreateManagedWidget("text_list", asciiTextWidgetClass, parent,
				       arg, i);
     
     XtAppAddActions(appCtx,sound_actionTable,1);
     trans=XtParseTranslationTable(setsoundtrans);
     XtOverrideTranslations(text_list,trans);
   }
   else {
     XtUnmapWidget(text_list);
     i = 0;
     XtSetArg(arg[i],XtNstring, text_str);   i++;
     XtSetValues( text_list, arg, i);
     XtMapWidget(text_list);
   }
}


/*
	Build the entry strings for the text widget.
*/
void	make_text_sound(construct_text_widget)
     Boolean	construct_text_widget;
{
  Widget  text_sink;
  Arg     arg[10];
  int     *tabs;
  XFontStruct	*font;
  
  int	i, cnt;
  int	max_window = 0;
  int	max_sound = 0;
  int	max_index = 0;
  int	max_volume = 0;
  char	vol[VOLLENGTH+1];
  
  font = (XFontStruct *) XLoadQueryFont( dpy, label_font_string);
  if ( font == NULL )
    printf ("font is null\n");
  
  free (sound_text_string); 
  
  sound_text_string = NULL;
  
  if ( num_sounds == 0 ) {
    sound_text_string = (char *) calloc( strlen(Empty_String), 1);
    strcpy(sound_text_string,Empty_String);
  } else
    
    for (i=0;i<num_sounds; i++) {
      if ( sound_text_string == NULL )
	sound_text_string = (char *) calloc( strlen(sounds[i]->window) +
					    INDEXLENGTH +
					    strlen(sounds[i]->sound) +
					    strlen(sounds[i]->volume) +
					    1 + 1 + 1 + 1, 1);
      else
	sound_text_string = (char *) realloc( sound_text_string,
					     strlen(sound_text_string) +
					     INDEXLENGTH +
					     strlen(sounds[i]->window) +
					     strlen(sounds[i]->sound) +
					     VOLLENGTH + 1 + 1 + 1 + 1);
      
      strcat(sound_text_string, sounds[i]->index);
      strcat(sound_text_string, "\t");
      strcat(sound_text_string, sounds[i]->window);
      strcat(sound_text_string, "\t");
      strcat(sound_text_string, sounds[i]->sound);
      strcat(sound_text_string, "\t");
      
      for (cnt = VOLLENGTH-strlen(sounds[i]->volume); cnt; cnt--)
	strcat(sound_text_string, " ");
      
      strcat(sound_text_string, sounds[i]->volume);
      strcat(sound_text_string, "\n");
      
      if ( strlen(sounds[i]->window) > max_window )
	max_window = strlen(sounds[i]->window);
      
      if ( strlen(sounds[i]->sound) > max_sound )
	max_sound = strlen(sounds[i]->sound);
      
      if ( strlen(sounds[i]->index) > max_index )
	max_index = strlen(sounds[i]->index);
      
      if ( strlen(sounds[i]->volume) > max_volume )
	max_volume = strlen(sounds[i]->volume);
      
    }
  
  if ( construct_text_widget ) {
    i = 0;
    XtSetArg(arg[i],XtNy,15);       i++;
    XtSetArg(arg[i],XtNheight,500);       i++;
    XtSetArg(arg[i],XtNwidth, 800);       i++;
    XtSetArg(arg[i],XtNstring, sound_text_string);   i++;
    XtSetArg(arg[i],XtNresize, XawtextResizeWidth);       i++;
    XtSetArg(arg[i],XtNselectTypes, (XtArgVal)memselect);   i++;
    XtSetArg(arg[i], XtNfromVert, quit);	i++;
    text_sound = XtCreateManagedWidget("text_sound", asciiTextWidgetClass, main_box,
				       arg, i);
    
    XtAppAddActions(appCtx,actionTable,2);
    trans=XtParseTranslationTable(setnametrans);
    XtOverrideTranslations(text_sound,trans);
  }
  else {
    XtUnmapWidget(text_sound);
    i = 0;
    XtSetArg(arg[i],XtNstring, sound_text_string);   i++;
    XtSetValues( text_sound, arg, i);
    XawTextUnsetSelection(text_sound);
    XtMapWidget(text_sound);
  }
  
  
  tabs = (int *) malloc (3 * sizeof(int));
  *tabs = INDEXLENGTH + 2;
  *(tabs+1) = max_window + 2 + *tabs;
  *(tabs+2) = max_sound + 2 + *(tabs+1);
  
  XtSetArg(arg[0],XtNtextSink,&text_sink);
  XtGetValues(text_sound, arg, ONE);
  if (text_sink == NULL)
    fprintf(stdout,"textSink widget not found");
  else
    XawTextSinkSetTabs(text_sink, 3, tabs);
}

/*
	This is called when the DELETE SOUND buttton is pressed.
	Subsequently pressing a button over an entry will remove it.
*/
void
remove_sound(index)
     int	index;
{
  String  color = "white";
  Pixel	pixel;
  Arg	arg[4];
  
  if ( (pixel = ConvertColor(delete_sound, color)) == -1 )
    fprintf(stderr,
	    "Cannot convert color: %s\n", color);
  else {
    int i = 0;
    XtSetArg(arg[i],XtNbackground, (Pixel) pixel); i++;
    XtSetValues(delete_sound, arg, i);
    
    free(sounds[index]->window);	sounds[index]->window = NULL;
    free(sounds[index]->directory);	sounds[index]->directory = NULL;
    free(sounds[index]->sound);	sounds[index]->sound = NULL;
    free(sounds[index]->volume);	sounds[index]->volume = NULL;
    free(sounds[index]->index);	sounds[index]->index = NULL;
    free(sounds[index]);		sounds[index] == NULL;
    
    for (i=index; i<num_sounds-1;i++) {
      sounds[i] = sounds[i+1];
      free(sounds[i]->index);
      sounds[i]->index = NULL;
      sounds[i]->index = (char *) calloc( strlen(num_to_str(i)),1 );
      strcpy(sounds[i]->index,num_to_str(i));
    }
    num_sounds--;
    make_text_sound(False);
    delete_snd = False;
  }
}


/*
	Copy the selected sound from the text widget 
	to the sound dialog box.
*/
void
display_sound(widget,cl,ca)
     Widget widget;
     caddr_t cl,ca;
{
  char	line[LINELENGTH];
  int     n=LINELENGTH;
  int	begin, end;
  Arg	args[5];
  Widget  tw = XtNameToWidget(widget,"*value");
  
  XawTextGetSelectionPos(text_list, &begin, &end);
  strcpy(line,XFetchBytes(XtDisplay(widget),&n));
  
  XtSetArg(args[0], XtNvalue, (XtArgVal) line);
  XtSetValues(sound_dialog, args,1);
  move_pt(sound_dialog, strlen(line));
  change_width(sound_dialog, line);
  
  read_data();
}

/*
  Move the caret to the end of the word in the dialog box.
*/
void	move_pt(widget, pos)
     Widget	widget;
     int	pos;
{
  Widget	text = XtNameToWidget(widget,"*value");
  
  if ( text !=  NULL)
    XawTextSetInsertionPoint(text, pos);
}

/*
	Make the dialog box as long the text in it.
*/
void	change_width(widget, str)
     Widget  widget;
     char	str[];
{
  Arg	args[4];
  Widget  text = XtNameToWidget(widget,"*value");
  XFontStruct *font;
  
  XtSetArg( args[0], XtNfont, &font);
  XtGetValues( text, args, ONE );
  if ( font == NULL )
    printf ("font is null\n");
  else {
    int	width;
    width = XTextWidth(font, str, strlen(str));
    XtSetArg( args[0], XtNwidth, width+10);
    XtSetValues( text, args, 1);
  }
}

/*
	When the mouse is clicked on an entry,
*/
void
display_set_members(w,cl,ca)
     Widget w;
     caddr_t cl,ca;
{
  char line[LINELENGTH];
  char str[INDEXLENGTH];
  int	n=LINELENGTH;
  int begin, end;
  int	index, pos;
  
  Arg	args[3];
  Widget	text;
  XFontStruct *font;
  
  font = (XFontStruct *) XLoadQueryFont( dpy, label_font_string);
  if ( font == NULL )
    printf ("font is null\n");
  
  XawTextGetSelectionPos(text_sound, &begin, &end);
  strcpy(line,XFetchBytes(XtDisplay(w),&n));
  
  if ( !strcmp(line,Empty_String) )
    return;
  
  strcpy(str, (char *)strtok(line," \t"));
  curindex = index = str_to_num(str);
  
  if ( delete_snd  && begin != end ) {
    remove_sound(index);
    return;
  }
  
  change_width(sound_dir, main_dir);
  move_pt(sound_dir, strlen(main_dir));
  
  XtSetArg(args[0], XtNvalue, sounds[index]->index);
  XtSetValues(index_dialog, args, 1);
  move_pt(index_dialog, strlen(sounds[index]->index));
  /*	change_width(index_dialog, sounds[index]->index);*/
  
  XtSetArg(args[0], XtNvalue, sounds[index]->window);
  XtSetValues(window_dialog, args, 1);
  move_pt(window_dialog, strlen(sounds[index]->window));
  change_width(window_dialog, sounds[index]->window);
  
  XtSetArg(args[0], XtNvalue, sounds[index]->sound);
  XtSetValues(sound_dialog, args, 1);
  move_pt(sound_dialog, strlen(sounds[index]->sound));
  change_width(sound_dialog, sounds[index]->sound);
  
  XtSetArg(args[0], XtNvalue, sounds[index]->volume);
  XtSetValues(volume_dialog, args, 1);
  move_pt(volume_dialog, strlen(sounds[index]->volume));
  /*change_width(volume_dialog, sounds[index]->volume);*/
  
  XtPopup(popup, XtGrabExclusive);
  
}

/*
	Free the storage and quit.
*/
void
quit_proc(w,cl,ca)
{
  int	i;
  
  for (i=0; i<num_sounds;i++) {
    free(sounds[i]->window);
    free(sounds[i]->directory);
    free(sounds[i]->sound);
    free(sounds[i]->volume);
    free(sounds[i]->index);
    free(sounds[i]);
  }
  XtDestroyApplicationContext (XtWidgetToApplicationContext(w));
  exit(0);
}


/*
	Write the data to the audio_specs file.
*/
void
save_sound_proc(w,cl,ca)
     Widget w;
     caddr_t cl,ca;
{
  int	i;
  char    *dir;
  FILE    *fp;
  
  if ((fp = fopen(audio_specs_file,"w"))==NULL)
    fprintf(stderr,"unable to open %s\n", audio_specs_file);
  else {
    for (i=0; i<num_sounds;i++) {
      fprintf(fp, "%s\t%s\t%s\n", sounds[i]->window,
	      sounds[i]->directory, sounds[i]->volume);
    }
    fclose(fp);
  }
}


/*
	Call the play_it() routine to play the sound passed.
*/
void
play_sound(volume, dir)
     char	*volume;
     char	*dir; 		/* the sound to play */
{
  int     argc = 0;       /* number of args */
  char    *argv[10];      /* arguments */
  char    audio_file[75]; /* used in file read */
  char    *vol_str = "-v";/* used in file read */
  int	i = 0;
  
  argv[i] = "playit";      i++;
  argv[i] = vol_str;       i++;
  argv[i] = volume;        i++;
  argv[i] = dir;		 i++;
  argv[i] =  "--";
  playit(i, argv);
}


/*
	Get the parameters to play the chosen sound.
*/
void
play_sound_proc(w,cl,ca)
     Widget w;
     caddr_t cl,ca;
{
  
  String sound_str = XawDialogGetValueString(sound_dialog);
  String volume_str = XawDialogGetValueString(volume_dialog);
  char	*dir;
  
  if (chk_volume( &volume_dialog, &volume_str ) &&
      chk_sound( &sound_dialog, &sound_str ) ) {
    
    dir = (char *) calloc( strlen(main_dir) + 1 +
			  strlen(sound_str), 1);
    strcpy(dir,main_dir);
    strcat(dir,"/");
    strcat(dir, sound_str);
    play_sound(volume_str, dir);
    free(dir);
  }
  
}


/*
	Play the currently selected sound.
*/
void
play_selection(w,cl,ca)
     Widget w;
     caddr_t cl,ca;
{
  char	line[LINELENGTH];
  int 	n=LINELENGTH;
  int	index;
  char	str[INDEXLENGTH];
  int	begin, end;
  
  XawTextGetSelectionPos(text_sound, &begin, &end);
  strcpy(line,XFetchBytes(XtDisplay(w),&n));
  
  if ( !strcmp(line,Empty_String) )
    return;
  
  strcpy(str, (char *)strtok(line," \t"));
  strcpy(str, (char *)strtok(line," \t"));
  index = str_to_num(str);
  if ( delete_snd  && begin != end ) {
    remove_sound(index);
    return;
  }
  play_sound(sounds[index]->volume, sounds[index]->directory);
}


/*
	Parse the window, sound, and volume from the entry chosen
	in the text widget.
*/
int	read_sound_file()
{
FILE    *fp;
char    wind[WINDLENGTH], dir[DIRLENGTH], vol[VOLLENGTH+1], line[LINELENGTH];
int     cnt = 0;
Sound_struct	*p;

  if ((fp = fopen(audio_specs_file,"r")) == NULL) {
    fprintf(stderr,"unable to open %s\n", audio_specs_file);
  }
  else {
    fprintf(stdout,"opened %s\n", audio_specs_file);
    while ( fgets(line,500,fp) ) {
      sscanf(line,"%s %s %s",wind, dir, vol);
      
      p = (Sound_struct *) malloc( sizeof(Sound_struct) );
      
      p->window = (char *) malloc( strlen(wind) );
      strcpy(p->window, wind);
      
      p->directory = (char *) malloc( strlen(dir) );
      strcpy(p->directory, dir);
      
      p->volume = (char *) malloc( strlen(vol) );
      strcpy(p->volume, vol);
      
      p->sound = (char *) malloc( strlen(find_sound(dir)) );
      strcpy(p->sound, (char *) find_sound(dir));
      
      p->index = (char *) malloc( strlen(num_to_str(cnt)) );
      strcpy(p->index,num_to_str(cnt));
      
      sounds[cnt] = p;
      
      cnt++;
    }
  }
  fclose(fp);
  return cnt;
}


/*
	Parse the filename out of the entire pathname.
*/
char	*find_sound(str)
     char	str[];
{
  char	slash = '/';
  char	*index = NULL;
  
  index = rindex(str, slash);
  if ( index == NULL ) 
    return (NULL);
  index++;
  return (index);
}

/*
	Read the sound filenames out of the specified directory.
*/
int	read_dir(entries, directory)
     char	*entries[];
     char	*directory;
{
  int     i = 0;
  int     cnt = 0;
  DIR	*dirp;
  int	main_entries;
  struct	dirent	*dp;
  int	c = 0;
  char	*p;
  char	*lf = "\n";
  
  if ( (dirp = opendir(directory)) == NULL )
    return -1;
  for (i=0, dp=readdir(dirp); dp != NULL; ++i, dp = readdir(dirp)) {
    if ( i >= MAXFILES ) {
      closedir(dirp);
      return -1;
    }
    else if ( (p = (char *) malloc( strlen( dp->d_name))) == NULL) {
      closedir(dirp);
      return -1;
    }
    else {
      strcpy(p, dp->d_name);
      entries[i] = p;
    }
  }
  closedir(dirp);
  return i;
}

void	sort(entries, number)
     char	*entries[];
     int	number;
{
  int	gap, i,j;
  char	*temp;
  
  for (gap = number/2; gap>0; gap /= 2)
    for (i=gap; i<number; i++)
      for (j= i-gap; j>= 0; j -= gap) {
	if (strcmp(entries[j], entries[j+gap]) <= 0)
	  break;
	temp = entries[j];
	entries[j] = entries[j+gap];
	entries[j+gap] = temp;
      }
}
