/*
	UnWindows 1.1
	change_colors.c

	Richard L. Kline
	Rensselaer Polytechnic Institute
	16 May 1994

	This is the color selection interface to the coloreyes
	program.  It allows the user to choose the
	colors he or she wants to associate with the distance
	indication given by coloreyes.
	Any number of colors from one to ten can be chosen.
	
	---
	Modification history:
	1.1 16 May 94 - all colors are now private read/write cells,
			preventing the program from chewing up all
			available colors on the display and allowing
			the eyes to immediately reflect new settings
*/
	
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Cardinals.h>
#include <math.h>
#include <X11/Shell.h>
#include <X11/Xaw/Scrollbar.h>
#include "Eyes.h"

char xeyes_specs_file[128];

void fds();
int	read_data();
XtEventHandler  change_color();
XtCallbackProc	add_color_proc();
XtCallbackProc	delete_color_proc();
XtCallbackProc	save_colors_proc();
XtCallbackProc	realign_colors_proc();
XtCallbackProc	done_proc();
XtCallbackProc  rgb_scroll();

Display * dpy;
Colormap cmap;
XtAppContext appCtx;

#define INIT_INTENS_RED		0.5
#define INIT_INTENS_GREEN	0.5
#define INIT_INTENS_BLUE	0.99
#define INIT_COLOR_STRING	"#8080ff"

#define INTENS(x)	((int)((x)*255))
#define RED   101
#define GREEN 102
#define BLUE  103
#define SPOTS 51.0
#define SCROLL_INC (1.0/SPOTS)

static  Widget          toplevel;
static	Widget		done;
static	Widget		add_color;
static	Widget		delete_color;
static	Widget		save_colors;
static	Widget		realign_colors;

static	Widget		red_scroll, green_scroll, blue_scroll;
static	Widget		red_label, green_label, blue_label;
static	Widget		color_box;

String		label_font_string = "-b&h-lucidatypewriter-bold-r-normal-sans-*-350-*-*-*-*-iso8859-*";

XFontStruct	*label_font;

char		colors[800][50];
int		color_index = 0;
extern EyesColorRec dis_colors[];	 /* variable number of color-% pairs */
extern int	num_colors;
Boolean		delete_pane = False;
String		example = "Hi There";

Pixel		rwpixel;	/* color for slider bars' output */
Pixel		boxpix[10];	/* colors for eyes and colorbox */
char		rwcolor[20] = INIT_COLOR_STRING;

Widget	settingsWindow = NULL;

Widget	pane;
Widget	modify_box;
Widget 	main_box;

static int pane_ctr = 0;
static int pane_num[10];

void init_settings (parent_dpy, parent_cmap, parent)
     Display * parent_dpy;
     Colormap parent_cmap;
     Widget parent;
{
  Widget choosefont_box;
  Widget box1;
  Widget eyes_box;
  XColor xcolor;
  int retval;
  
  int	argc_int = 0;
  char	*argv_int[10];
  
  Arg	arg[10];

  toplevel = parent;
  dpy = parent_dpy;
  cmap = parent_cmap;

  strcpy ( xeyes_specs_file, getenv ( "HOME" ) );
  strcat ( xeyes_specs_file, "/.coloreyes" );
  
  /* a box to hold everything */	
  settingsWindow = XtVaCreatePopupShell ("settings",
				      transientShellWidgetClass, parent,
				      XtNheight, 490, XtNwidth, 830,
				      XtNtitle, "Coloreyes Settings", NULL);
  main_box = XtVaCreateManagedWidget ("main_box", compositeWidgetClass,
				      settingsWindow,
				      XtNheight, 490, XtNwidth, 830, NULL);
  
  /* the REALIGN COLORS button */
  realign_colors = XtVaCreateManagedWidget("Realign Colors",commandWidgetClass,
				  main_box, XtNx, 5, XtNy, 30, NULL);
  XtAddCallback (realign_colors, XtNcallback, realign_colors_proc, NULL);
  
  /* the DELETE COLOR button */
  delete_color = XtVaCreateManagedWidget ("Delete Color", commandWidgetClass,
				  main_box, XtNx, 5, XtNy, 100, NULL);
  XtAddCallback (delete_color, XtNcallback, delete_color_proc, NULL);
  
  /* the ADD COLOR button */
  add_color = XtVaCreateManagedWidget ("Add Color", commandWidgetClass,
				  main_box, XtNx, 5, XtNy, 170, NULL);
  XtAddCallback (add_color, XtNcallback, add_color_proc, NULL);
  
  /* the SAVE COLORS button */
  save_colors = XtVaCreateManagedWidget ("Save settings", commandWidgetClass,
				  main_box, XtNx, 5, XtNy, 430, NULL);
  XtAddCallback (save_colors, XtNcallback, save_colors_proc, NULL);
  
  /* the DONE button */
  done = XtVaCreateManagedWidget ("Done", commandWidgetClass, main_box,
				  XtNx, 730, XtNy, 430, NULL);
  XtAddCallback (done, XtNcallback, done_proc, NULL);
  
  /* a box to show current color selection */
  
  color_box = XtVaCreateManagedWidget ("color_box", compositeWidgetClass,
				    main_box, XtNx, 5, XtNy, 250,
				    XtNwidth, 150, XtNheight, 150, NULL);
  
  
  /* a box of colors that the eyes become */
  
  pane = XtVaCreateManagedWidget("Paned", panedWidgetClass, main_box,
				 XtNx, 320, XtNy, 10, XtNwidth, 500,
				 XtNheight, 200, XtNorientation,
				 XtorientHorizontal, NULL);

  /* Get a private read/write color cell for the slider bars. */
  XAllocColorCells (dpy, cmap, /*contiguous*/ False, /*plane_masks*/ NULL,
		    /*n_planes*/ 0, /*pixel_array*/ &rwpixel, /*n_colors*/ 1);
  /* Get ten color cells for the eyes and the paned color box. */
  retval = XAllocColorCells (dpy, cmap, False, NULL, 0, boxpix, 10);
  if (!retval) {
    fprintf (stderr, "coloreyes: could not allocate enough read/write color\
 cells!\n           close a color-intensive application and restart.\n\n");
    exit (1);
  }

  /* Store initial color for slider bar box. */
  XParseColor (dpy, cmap, INIT_COLOR_STRING, &xcolor);
  xcolor.pixel = rwpixel;
  XStoreColor (dpy, cmap, &xcolor);

  fds(settingsWindow,main_box);
  
  num_colors = read_data();
  
}

/*
	Converts a color's name to its value.
*/

static int
ConvertColor(w, color_name)
Widget w;
char * color_name;
{
  XrmValue from, to;

  from.size = strlen(color_name) + 1;
  from.addr = color_name;

/*
 * This conversion accepts a colorname from rgb.txt, or a #rrrgggbbb
 * rgb color definition, or the special toolkit strings "XtDefaultForeground"
 * and "XtDefaultBackground".
 */

  XtConvert(w, XtRString, (XrmValuePtr) &from, XtRPixel, (XrmValuePtr) &to);
  if (to.addr == NULL) {
      return(-1);
  }

  return( (int) *((Pixel *) to.addr) );
}


/*
	This procedure is called when the ADD_COLOR button is pressed.
	It adds another panel to the pane widget.  The color of the panel 
	is the currently selected color.
*/

XtCallbackProc add_color_proc(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
  Arg     args[10];
  Arg     gargs[5];
  int     pane_height, box_height;
  int     pane_width, box_width;
  int     box_x;
  Widget  box;
  XColor  xcolor;
  int	cnt;
  int	i;
  WidgetList	children;
  int	numchildren;
  int	numpanes;
  
  if ( num_colors == 10 ) {
    fprintf (stderr,"coloreyes: you cannot add another color\n");	
    return;
  }
  
  XtSetArg(gargs[0],XtNheight,0);
  XtSetArg(gargs[1],XtNwidth,0);
  XtGetValues(pane,gargs,2);
  pane_height = (int)(gargs[0].value);
  pane_width = (int)(gargs[1].value);
  if ( num_colors == 0 )
    box_width = pane_width;
  else
    box_width = pane_width / num_colors;
  
  cnt = 0;

  XParseColor (dpy, cmap, rwcolor, &xcolor);
  xcolor.pixel = boxpix[num_colors];
  XStoreColor (dpy, cmap, &xcolor);

  XtSetArg (args[cnt], XtNbackground, boxpix[num_colors]); cnt++;
  XtSetArg (args[cnt], XtNpreferredPaneSize, box_width);   cnt++;

  box = XtCreateManagedWidget("box", boxWidgetClass,pane,args,cnt);
  XtAddEventHandler (box, ButtonPressMask, FALSE, change_color,
		     (XtPointer)pane_ctr);
  pane_num[num_colors] = pane_ctr++;
  
  XtSetArg (gargs[0], XtNnumChildren, &numchildren);
  XtSetArg (gargs[1], XtNchildren, &children);
  XtGetValues (pane, gargs, 2);
  
  numpanes = XawPanedGetNumSub (pane);
  box_width = pane_width / numpanes;
  
  XawPanedSetRefigureMode (pane, False);
  for (i=0; i < numchildren; i++) {
    if (strcmp (XtName (children[i]), "box") == 0) {
      cnt = 0;
      XtSetArg (args[cnt], XtNpreferredPaneSize, box_width); cnt++;
      XtSetArg (args[cnt], XtNwidth, box_width); cnt++;
      XtSetValues (children[i], args, cnt);
    }
  }
  
  XawPanedSetRefigureMode(pane,True);
  XtManageChildren(children, numchildren);

  num_colors++;

  return;
  
}



/* called when the REALIGN COLORS button is pressed

	- this procedure divides the real estate equally among the
	  colors.

	- For example, if there are two colors, red and blue, and
	  red has 75% of the real estate and blue the other 25%,
	  pressing REALIGN COLORS will give each 50%.
*/

XtCallbackProc realign_colors_proc(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
  Arg     args[10];
  Arg     gargs[5];
  int     pane_height, box_height;
  int     pane_width, box_width;
  int     box_x;
  Widget  box;
  int     pixel;
  int     cnt;
  int     i;
  WidgetList      children;
  int     numchildren;
  int     numpanes;
  
  if ( num_colors == 10 ) {
    fprintf (stderr,"You cannot add another color\n");
    return;
  }
  
  if ( num_colors < 2 )
    return;

  XtSetArg(gargs[0],XtNheight,0);
  XtSetArg(gargs[1],XtNwidth,0);
  XtGetValues(pane,gargs,2);
  pane_height = (int)(gargs[0].value);
  pane_width = (int)(gargs[1].value);
  
  if ( XawPanedGetNumSub(pane) == 0 )
    box_width = pane_width;
  else
    box_width = pane_width / XawPanedGetNumSub(pane);
  
  XtSetArg(gargs[0],XtNnumChildren, &numchildren);
  XtSetArg(gargs[1],XtNchildren, &children);
  XtGetValues(pane,gargs,2);
  
  numpanes = XawPanedGetNumSub(pane);
  box_width = pane_width / numpanes;
  
  XawPanedSetRefigureMode(pane,False);
  for( i=0; i < numchildren; i++) {
    if ( strcmp(XtName(children[i]), "box") == 0) {
      cnt = 0;
      XtSetArg(args[cnt], XtNpreferredPaneSize, box_width); cnt++;
      XtSetArg(args[cnt], XtNwidth, box_width); cnt++;
      XtSetValues(children[i],args,cnt);
    }
  }
  
  XawPanedSetRefigureMode(pane,True);
  XtManageChildren(children, numchildren);
  return;
  
}


/*
	This is called when the DELETE button is pressed. The boolean
	delete_pane is set to True.  Subsequent clicking in a pane will
	check this variable and delete the pane if it is set.  Otherwise,
	the color of the panel will be changed to the currently selected 
        color.  See the change_color procedure.

	Pressing the button when delete_pane is True removes
	the button highlight and sets delte_pane to False, effectively
	cancelling the delete function.
*/
XtCallbackProc delete_color_proc(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
  Arg     args[10];
  String	selected_color = "gray71";
  int	pixel;
  int	i;
  int which = (int) call_data;
  
  if (delete_pane == False && num_colors > 1) {

    if ( (pixel = ConvertColor(delete_color, selected_color)) == -1 )
      fprintf(stderr,"Cannot convert color: %s\n", selected_color);
    else {
      i = 0;
      XtSetArg(args[i],XtNbackground, (Pixel) pixel); i++;
      XtSetValues(widget, args, i);
    }
    delete_pane = True;

  } else {

    XtVaGetValues (done, XtNbackground, &pixel, NULL);
    XtVaSetValues (widget, XtNbackground, pixel, NULL);
    delete_pane = False;

  }
}



/*
	This is called when the SAVE button is pressed.  The
	data is written to the xeyes_specs file.
*/
XtCallbackProc save_colors_proc(widget, closure, xevent)
Widget widget;
XtPointer closure;
XEvent  *xevent;
{
  Arg     args[10];
  Arg     gargs[5];
  int     pane_height, box_height;
  int     pane_width, box_width;
  int     box_x;
  Widget  box;
  int     cnt;
  int     i;
  WidgetList      children;
  int     numchildren;
  int     numpanes;
  float	offsets[10];
  int	off_index = 0;
  char	colorname[10][20];
  int	cindex = 0;
  FILE    *fp;
  
  XtSetArg(gargs[0],XtNheight,0);
  XtSetArg(gargs[1],XtNwidth,0);
  XtGetValues(pane,gargs,2);
  pane_height = (int)(gargs[0].value);
  pane_width = (int)(gargs[1].value);
  
  XtSetArg(gargs[0],XtNnumChildren, &numchildren);
  XtSetArg(gargs[1],XtNchildren, &children);
  XtGetValues(pane,gargs,2);
  
  numpanes = XawPanedGetNumSub(pane);
  
  for( i=0; i < numchildren; i++) {
    if ( strcmp(XtName(children[i]), "box") == 0) {
      XrmValue from, to;
      int	pixel;
      int	x, y;
      int	width;
      int	red, green, blue;
      int	fred, fgreen, fblue;
      XColor	xcolor;
      char    junk[35], line[80];
      float	loffset, uoffset;
      
      cnt = 0;
      XtSetArg(gargs[cnt], XtNbackground, 0); cnt++;
      XtSetArg(gargs[cnt], XtNx, 0); cnt++;
      XtSetArg(gargs[cnt], XtNy, 0); cnt++;
      XtSetArg(gargs[cnt], XtNwidth, 0); cnt++;
      XtGetValues(children[i],gargs,cnt);
      
      x = (int)(gargs[1].value);
      y = (int)(gargs[2].value);
      width = (int)(gargs[3].value);
      offsets[off_index] = (float) 1.0 * ( (1.0*x)/ (1.0*pane_width) );
      off_index++;
      
      xcolor.pixel = (int)(gargs[0].value);
      XQueryColor (dpy, cmap, &xcolor);
      red = xcolor.red >> 8;
      green = xcolor.green >> 8;
      blue = xcolor.blue >> 8;
      
      sprintf ( colorname[i], "rgb:%02x/%02x/%02x", red, green, blue );

    }
  }
  
  if ((fp = fopen(xeyes_specs_file,"w"))==NULL)
    fprintf(stderr,"unable to open %s\n",xeyes_specs_file);
  else {
    for (cnt=0; cnt<off_index; cnt++) {
      if ( cnt == 0 )
	fprintf(fp, "%4.2f\t%4.2f\t%d\t%s\n", offsets[cnt],
		offsets[cnt+1], cnt+100, colorname[cnt]);
      else	
	if ( cnt < off_index-1 )
	  fprintf(fp, "%4.2f\t%4.2f\t%d\t%s\n", offsets[cnt]+0.01,
		  offsets[cnt+1], cnt+100, colorname[cnt]);
	else
	  fprintf(fp, "%4.2f\t%4.2f\t%d\t%s\n", offsets[cnt]+0.01,
		  1.0, cnt+100, colorname[cnt]);
    }
    fclose(fp);
  }
}


/* called when the DONE button is pressed. */

XtCallbackProc done_proc(widget, closure, call_data)
Widget widget;
XtPointer closure, call_data;
{
  Arg     gargs[5];
  int     pane_height, box_height;
  int     pane_width, box_width;
  int     box_x;
  Widget  box;
  int     cnt;
  int     i;
  WidgetList      children;
  int     numchildren;
  int     numpanes;
  float	  offset[10];
  
  XtSetArg(gargs[0],XtNheight,0);
  XtSetArg(gargs[1],XtNwidth,0);
  XtGetValues(pane,gargs,2);
  pane_height = (int)(gargs[0].value);
  pane_width = (int)(gargs[1].value);
  
  XtSetArg(gargs[0],XtNnumChildren, &numchildren);
  XtSetArg(gargs[1],XtNchildren, &children);
  XtGetValues(pane,gargs,2);
  
  numpanes = XawPanedGetNumSub(pane);
  
  for (i=0; i < numchildren; i++) {
    if ( strcmp(XtName(children[i]), "box") == 0) {
      int	x, y;
      int	width;
      float	loffset, uoffset;
      
      cnt = 0;
      XtSetArg(gargs[cnt], XtNx, 0); cnt++;
      XtGetValues(children[i],gargs,cnt);
      
      x = (int)(gargs[0].value);
      offset[i] = (float) 1.0 * ( (1.0*x)/ (1.0*pane_width) );
    }
  }
  
  for (i=1; i<num_colors; i++) {
    dis_colors[i-1].eyes_uoffset = offset[i]-0.01;
    dis_colors[i].eyes_loffset   = offset[i];
  }
  dis_colors[num_colors].eyes_uoffset = 1.00;

  XtPopdown (settingsWindow);

}


/*
	This procedure is called when a button is pressed inside a 
	panel.  If the DELETE button has been previously pressed,
	the panel is deleted.  If not, the color of the panel is 
	changed to that of the current color as specified by the
 	scrollbar.  The same color is also in the color box, and
        its name is in the label widget.
*/

XtEventHandler change_color(widget, boxnum, xevent)
Widget widget;
int boxnum;
XEvent	*xevent;
{
  Arg     arg[10];
  int     i = 0;
  int	pixel;
  XColor  xcolor;
  XGCValues myXGCV;
  WidgetList pane_child;

  if ( !num_colors )
    return;

  if ( !delete_pane ) { /* replace color */

    XParseColor (dpy, cmap, rwcolor, &xcolor);
    xcolor.pixel = boxpix[boxnum];
    XStoreColor (dpy, cmap, &xcolor);
    i = 0;
    XtSetArg(arg[i],XtNbackground, xcolor.pixel); i++;
    XtSetValues(widget, arg, i);

  } else { /* delete color */

    String  color = "white";
    if ( (pixel = ConvertColor(delete_color, color)) == -1 )
      fprintf(stderr,
	      "Cannot convert color: %s\n", color);
    else {
      i = 0;
      XtSetArg(arg[i],XtNbackground, (Pixel) pixel); i++;
      XtSetValues(delete_color, arg, i);
    }
    
    XawPanedSetRefigureMode (pane, False);
    XtDestroyWidget(widget);
    delete_pane = False;
    num_colors--;

    /* Logically shift remaining color pane information to the left. */
    XtVaGetValues (pane, XtNchildren, &pane_child, NULL);
    i = 0;
    while (pane_num[i] != boxnum) i++;
    while (i < num_colors) {
      pane_num[i] = pane_num[i+1];
      xcolor.pixel = boxpix[i+1];
      XQueryColor (dpy, cmap, &xcolor);
      xcolor.pixel = boxpix[i];
      XStoreColor (dpy, cmap, &xcolor);
      XtVaSetValues (pane_child[i], XtNbackground, boxpix[i-1], NULL);
      i++;
    }
    XawPanedSetRefigureMode (pane, True);

  }

} /* change_color */


XtCallbackProc rgb_scroll (w, client_data, position)
     Widget w;
     XtPointer client_data;
     XtPointer position;
{
  static double rintensity = INIT_INTENS_RED, 
                gintensity = INIT_INTENS_GREEN, 
                bintensity = INIT_INTENS_BLUE;
  double intensity;
  int color;
  XColor xcolor;
  Widget w_label;
  char ch[20];

  /* Figure out which of the r, g, b scrollbars this is */
  if (w == red_scroll) {
    color = RED;
    intensity = rintensity;
    w_label = red_label;
  } else if (w == green_scroll) {
    color = GREEN;
    intensity = gintensity;
    w_label = green_label;
  } else {
    color = BLUE;
    intensity = bintensity;
    w_label = blue_label;
  }

  if ((int)client_data == 1) { /* left or right button pressed */

    if ((int)position > 0) { /* left button was pressed */
      if (intensity >= SCROLL_INC)
        intensity = (int)(intensity * SPOTS + 0.001) / SPOTS - SCROLL_INC;
      else
        intensity = 0.0;
    } else { /* right button was pressed */
      if (intensity < (1-SCROLL_INC))
        intensity = (int)(intensity * SPOTS + 0.001) / SPOTS + SCROLL_INC;
      else
        intensity = 1-SCROLL_INC;
    }

  } else { /* middle button pressed */

    intensity = *((float *)position);

  }
  XawScrollbarSetThumb (w, intensity, SCROLL_INC);

  /* Update the appropriate color value and its label on the screen. */
  if (color == RED)
    rintensity = intensity;
  else if (color == GREEN)
    gintensity = intensity;
  else
    bintensity = intensity;
  sprintf (ch, "%d", (int) (intensity * 256) );
  XtVaSetValues (w_label, XtNlabel, ch, NULL);

  sprintf (rwcolor, "rgb:%02x/%02x/%02x", (int) (rintensity * 255.9),
	   (int) (gintensity * 255.9), (int) (bintensity * 255.9) );
  XParseColor (dpy, cmap, rwcolor, &xcolor);
  xcolor.pixel = rwpixel;
  XStoreColor (dpy, cmap, &xcolor);
 
}


/*
	This procedure reads the xeyes_specs file to initialize
	the paned widget.  Each panel in the widget is assigned
	a color.  The user can change the colors of panels, add
	new panels, or delete panels.
	Returns the number of colors read in.
*/

int	read_data()
{
  int     argc = 0;       /* number of args */
  char    *argv[10];      /* arguments */
  char    color[45];      /* used in file read */
  char    line[80];       /* used in file read */
  char    junk[35];       /* used in file read */
  float   loffset;        /* used in file read */
  float   uoffset;        /* used in file read */
  int     number;         /* used in file read */
  FILE    *fp;            /* used in file read */
  int     i = 0;
  int     cnt = 0;
  XColor  xcolor;
  
  Arg	args[10];
  Arg	gargs[5];
  int	pane_height, box_height;
  int	pane_width, box_width;
  int	box_x;
  Widget	box;
  int     pixel;
  int   defaults = 0;
  int   done = 0;
  
  
  XtSetArg(gargs[0],XtNheight,0);
  XtSetArg(gargs[1],XtNwidth,0);
  XtGetValues(pane,gargs,2);
  pane_height = (int)(gargs[0].value);
  pane_width = (int)(gargs[1].value);
  
  if ((fp = fopen(xeyes_specs_file,"r")) == NULL) {
      /*   fprintf(stderr,"unable to open %s\n", xeyes_specs_file);*/
    defaults = 1;
  }

  while ( !done ) {
    if (!defaults) {
      
      if (fgets(line,80,fp) ) {
	sscanf (line,"%f %f %d",&loffset, &uoffset, &number);
	strcpy (junk, strtok(line," \t") );
	strcpy (junk, strtok(NULL," \t"));
	strcpy (junk, strtok(NULL," \t"));
	strcpy (color, strtok(NULL,"\t\n"));
      } else {
	done = 1;
      }
      
    } else if (defaults == 1) {
      
      strcpy (color, "red");
      loffset = 0;
      uoffset = 0.33;
      number = 100;
      defaults++;
      
    } else if (defaults == 2) {
      
      strcpy (color, "green4");
      loffset = 0.34;
      uoffset = 0.66;
      number = 101;
      defaults++;
      
    } else if (defaults == 3) {
      
      strcpy (color, "blue");
      loffset = 0.67;
      uoffset = 1;
      number = 102;
      done = 1;
      
    }
    
    if (defaults || !done) {

      box_width = pane_width * (uoffset - loffset);
      
      cnt = 0;

/*      if ( (pixel = ConvertColor(pane, color)) == -1 )
	fprintf(stderr,
		"Cannot convert color: %s\n", color);
      else
	XtSetArg(args[cnt],XtNbackground, (Pixel) pixel); cnt++;
*/
      XParseColor (dpy, cmap, color, &xcolor);
      xcolor.pixel = boxpix[i];
      XStoreColor (dpy, cmap, &xcolor);
      XtSetArg (args[cnt], XtNbackground, boxpix[i]);      cnt++;
      XtSetArg (args[cnt], XtNpreferredPaneSize, box_width); cnt++;

      /* make a box over each pane */
      box = XtCreateManagedWidget ("box", boxWidgetClass, pane, args, cnt);
      
      /* When a button is pressed in the box, change the color of the pane
	 to the color currently selected by the scrollbars and displayed
	 in the box to their left.      */
      XtAddEventHandler (box, ButtonPressMask, FALSE, change_color,
			 (XtPointer)pane_ctr);
      pane_num[i] = pane_ctr++;
      
      i++;
    }
  }

  if (!defaults)
    fclose(fp);
    
  return i;
  
}


/*
	This function automates the construction of a scrollbar.
*/	
void	setscroll (parent, scroll_widget, name, scroll_proc, scroll_data,
		   x, y)
     Widget		*parent;
     Widget		*scroll_widget;
     String		name;
     XtCallbackProc	(*scroll_proc)();
     double		scroll_data;
     int		x, y;
{
  
  *scroll_widget = XtVaCreateManagedWidget(name, scrollbarWidgetClass,
		      *parent, XtNx, x, XtNy, y, XtNwidth, 500, XtNheight, 45,
		      XtNorientation, XtorientHorizontal, NULL);
  XawScrollbarSetThumb (*scroll_widget, scroll_data, SCROLL_INC);
  XtAddCallback (*scroll_widget, XtNscrollProc, (*scroll_proc), (XtPointer)1);
  XtAddCallback (*scroll_widget, XtNjumpProc, (*scroll_proc), (XtPointer)2);

}

/*
	- load the font to display the colors' names
	- read the available colors from the system's color file
	- make a label widget the colors' names
	- make a scrollbar to riffle through the colors
*/
void	fds(topLevel, box)
Widget		topLevel;
Widget		box;
{
  Arg	arg[10];
  int	i;
  int	pixel;
  
  label_font = XLoadQueryFont( XtDisplay(topLevel), label_font_string);
  
  XtVaSetValues (color_box, XtNbackground, rwpixel, NULL);

  XtVaCreateManagedWidget ("red_letter", labelWidgetClass, box,
			   XtNlabel, "R:", XtNresize, False,
			   XtNborderWidth, 0, XtNx, 180, XtNy, 250, NULL);
  XtVaCreateManagedWidget ("green_letter", labelWidgetClass, box,
			   XtNlabel, "G:", XtNresize, False,
			   XtNborderWidth, 0, XtNx, 180, XtNy, 302, NULL);
  XtVaCreateManagedWidget ("blue_letter", labelWidgetClass, box,
			   XtNlabel, "B:", XtNresize, False,
			   XtNborderWidth, 0, XtNx, 180, XtNy, 354, NULL);
  
  red_label = XtVaCreateManagedWidget ("red_label", labelWidgetClass, box, 
				       XtNlabel, "128", XtNresize, False,
       		           XtNborderWidth, 0, XtNx, 235, XtNy, 250, NULL);
  green_label = XtVaCreateManagedWidget ("green_label", labelWidgetClass, box,
					 XtNlabel, "128", XtNresize, False,
			   XtNborderWidth, 0, XtNx, 235, XtNy, 302, NULL);
  blue_label = XtVaCreateManagedWidget ("blue_label", labelWidgetClass, box,
					XtNlabel, "255", XtNresize, False,
			   XtNborderWidth, 0, XtNx, 235, XtNy, 354, NULL);

  /* scroll bar to riffle through the colors */
  setscroll(&box, &red_scroll, "red_scroll", rgb_scroll,
	    INIT_INTENS_RED, 320, 250);
  setscroll(&box, &green_scroll, "green_scroll", rgb_scroll, 
	    INIT_INTENS_GREEN, 320, 302);
  setscroll(&box, &blue_scroll, "blue_scroll", rgb_scroll,
	    INIT_INTENS_BLUE, 320, 354);

}
